@@ -30,7 +30,7 @@
 #include "nautilus-directory-notify.h"
 #include "nautilus-file.h"
 #include <eel/eel-glib-extensions.h>
-#include <gtk/gtksignal.h>
+#include <gtk/gtk.h>
 #include <libgnome/gnome-macros.h>
 
 struct NautilusMergedDirectoryDetails {
@@ -559,7 +559,7 @@ real_directory_notify_files_removed (NautilusDirectory *real_directory)
 	}
 
 	if (files) {
-		nautilus_directory_notify_files_removed (files);
+		nautilus_directory_notify_files_removed_by_uri (files);
 	}
 
 	eel_g_list_free_deep (files);
