@@ -30,8 +30,8 @@
 #include <eel/eel-string.h>
 #include <eel/eel-mount-operation.h>
 #include <glib/gi18n.h>
+#include <glib/gstdio.h>
 #include <string.h>
-#include <unistd.h>
 
 #include "nautilus-file-attributes.h"
 #include "nautilus-file.h"
@@ -59,7 +59,8 @@ typedef struct {
 } ApplicationLaunchParameters;
 
 typedef struct {
-	NautilusWindowInfo *window_info;
+	NautilusWindowSlotInfo *slot_info;
+	gpointer window_info;
 	GtkWindow *parent_window;
 	GCancellable *cancellable;
 	GList *files;
@@ -72,6 +73,7 @@ typedef struct {
 	NautilusFileListHandle *files_handle;
 	gboolean tried_mounting;
 	char *activation_directory;
+	gboolean user_confirmation;
 } ActivateParameters;
 
 /* Number of seconds until cancel dialog shows up */
@@ -598,9 +600,9 @@ report_broken_symbolic_link (GtkWindow *parent_window, NautilusFile *file)
 
 	target_path = nautilus_file_get_symbolic_link_target_path (file);
 	if (target_path == NULL) {
-		detail = g_strdup (_("This link can't be used, because it has no target."));
+		detail = g_strdup (_("This link cannot be used, because it has no target."));
 	} else {
-		detail = g_strdup_printf (_("This link can't be used, because its target "
+		detail = g_strdup_printf (_("This link cannot be used, because its target "
 					    "\"%s\" doesn't exist."), target_path);
 	}
 	
@@ -910,8 +912,8 @@ activation_parameters_free (ActivateParameters *parameters)
 		eel_timed_wait_stop (cancel_activate_callback, parameters);
 	}
 	
-	if (parameters->window_info) {
-		g_object_remove_weak_pointer (G_OBJECT (parameters->window_info), (gpointer *)&parameters->window_info);
+	if (parameters->slot_info) {
+		g_object_remove_weak_pointer (G_OBJECT (parameters->slot_info), (gpointer *)&parameters->slot_info);
 	}
 	if (parameters->parent_window) {
 		g_object_remove_weak_pointer (G_OBJECT (parameters->parent_window), (gpointer *)&parameters->parent_window);
@@ -972,7 +974,9 @@ activate_mount_op_active (EelMountOperation *operation,
 }
 
 static gboolean
-confirm_multiple_windows (GtkWindow *parent_window, int count)
+confirm_multiple_windows (GtkWindow *parent_window,
+			  int count,
+			  gboolean use_tabs)
 {
 	GtkDialog *dialog;
 	char *prompt;
@@ -984,8 +988,13 @@ confirm_multiple_windows (GtkWindow *parent_window, int count)
 	}
 
 	prompt = _("Are you sure you want to open all files?");
-	detail = g_strdup_printf (ngettext("This will open %d separate window.",
-					   "This will open %d separate windows.", count), count);
+	if (use_tabs) {
+		detail = g_strdup_printf (ngettext("This will open %d separate tab.",
+						   "This will open %d separate tabs.", count), count);
+	} else {
+		detail = g_strdup_printf (ngettext("This will open %d separate window.",
+						   "This will open %d separate windows.", count), count);
+	}
 	dialog = eel_show_yes_no_dialog (prompt, detail, 
 					 GTK_STOCK_OK, GTK_STOCK_CANCEL,
 					 parent_window);
@@ -1000,6 +1009,8 @@ confirm_multiple_windows (GtkWindow *parent_window, int count)
 static void
 activate_files (ActivateParameters *parameters)
 {
+	NautilusWindowInfo *window_info;
+	NautilusWindowOpenFlags flags;
 	NautilusFile *file;
 	GList *launch_desktop_files;
 	GList *launch_files;
@@ -1081,7 +1092,7 @@ activate_files (ActivateParameters *parameters)
 	if (parameters->activation_directory &&
 	    (launch_files != NULL || launch_in_terminal_files != NULL)) {
 		old_working_dir = g_get_current_dir ();
-		chdir (parameters->activation_directory);
+		g_chdir (parameters->activation_directory);
 		
 	}
 
@@ -1127,20 +1138,39 @@ activate_files (ActivateParameters *parameters)
 	}
 
 	if (old_working_dir != NULL) {
-		chdir (old_working_dir);
+		g_chdir (old_working_dir);
 		g_free (old_working_dir);
 	}
 
 	open_in_view_files = g_list_reverse (open_in_view_files);
 	count = g_list_length (open_in_view_files);
-	if (parameters->window_info != NULL &&
-	    confirm_multiple_windows (parameters->parent_window, count)) {
-		NautilusWindowOpenFlags flags;
 
-		flags = parameters->flags;
-		if (count > 1) {
+	flags = parameters->flags;
+	if (count > 1) {
+		if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ENABLE_TABS) &&
+		    (parameters->flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW) == 0) {
+			flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB;
+		} else {
 			flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW;
 		}
+	}
+
+	if (parameters->slot_info != NULL &&
+	    (!parameters->user_confirmation ||
+	     confirm_multiple_windows (parameters->parent_window, count,
+				       (flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB) != 0))) {
+
+		if ((flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB) != 0 &&
+		    eel_preferences_get_enum (NAUTILUS_PREFERENCES_NEW_TAB_POSITION) ==
+		    NAUTILUS_NEW_TAB_POSITION_AFTER_CURRENT_TAB) {
+			/* When inserting N tabs after the current one,
+			 * we first open tab N, then tab N-1, ..., then tab 0.
+			 * Each of them is appended to the current tab, i.e.
+			 * prepended to the list of tabs to open.
+			 */
+			open_in_view_files = g_list_reverse (open_in_view_files);
+		}
+
 
 		for (l = open_in_view_files; l != NULL; l = l->next) {
 			GFile *f;
@@ -1150,8 +1180,8 @@ activate_files (ActivateParameters *parameters)
 
 			uri = nautilus_file_get_activation_uri (file);
 			f = g_file_new_for_uri (uri);
-			nautilus_window_info_open_location (parameters->window_info,
-							    f, parameters->mode, flags, NULL);
+			nautilus_window_slot_info_open_location (parameters->slot_info,
+								 f, parameters->mode, flags, NULL);
 			g_object_unref (f);
 			g_free (uri);
 		}
@@ -1196,7 +1226,7 @@ activate_files (ActivateParameters *parameters)
 			(full_uri_for_display, MAX_URI_IN_DIALOG_LENGTH);
 		g_free (full_uri_for_display);
 	
-		error_message = g_strdup_printf (_("Couldn't display \"%s\"."),
+		error_message = g_strdup_printf (_("Could not display \"%s\"."),
 						 uri_for_display);
 		
 		g_free (uri_for_display);
@@ -1207,12 +1237,17 @@ activate_files (ActivateParameters *parameters)
 		g_free (error_message);
 	}
 
+	window_info = NULL;
+	if (parameters->slot_info != NULL) {
+		window_info = nautilus_window_slot_info_get_window (parameters->slot_info);
+	}
+
 	if (open_in_app_parameters != NULL ||
 	    unhandled_open_in_app_files != NULL) {
 		if ((parameters->flags & NAUTILUS_WINDOW_OPEN_FLAG_CLOSE_BEHIND) != 0 &&
-		    parameters->window_info != NULL && 
-		     nautilus_window_info_get_window_type (parameters->window_info) == NAUTILUS_WINDOW_SPATIAL) {
-			nautilus_window_info_close (parameters->window_info);
+		    window_info != NULL && 
+		     nautilus_window_info_get_window_type (window_info) == NAUTILUS_WINDOW_SPATIAL) {
+			nautilus_window_info_close (window_info);
 		}
 	}
 
@@ -1460,6 +1495,7 @@ activation_mountable_mounted (NautilusFile  *file,
 {
 	ActivateParameters *parameters = callback_data;
 	NautilusFile *target_file;
+	int position;
 
 	/* Remove from list of files that have to be mounted */
 	parameters->mountables = g_list_remove (parameters->mountables, file); 
@@ -1467,13 +1503,16 @@ activation_mountable_mounted (NautilusFile  *file,
 
 	if (error == NULL) {
 		/* Replace file with the result of the mount */
-		
 		target_file = nautilus_file_get (result_location);
 		
+		position = g_list_index (parameters->files, file);
+
 		parameters->files = g_list_remove (parameters->files, file); 
 		nautilus_file_unref (file);
 		
-		parameters->files = g_list_prepend (parameters->files, target_file);
+		parameters->files = g_list_insert (parameters->files,
+						   target_file,
+						   position);
 	} else {
 		/* Remove failed file */
 		
@@ -1537,11 +1576,12 @@ activation_mount_mountables (ActivateParameters *parameters)
  **/
 void
 nautilus_mime_activate_files (GtkWindow *parent_window,
-			      NautilusWindowInfo *window_info,
+			      NautilusWindowSlotInfo *slot_info,
 			      GList *files,
 			      const char *launch_directory,
 			      NautilusWindowOpenMode mode,
-			      NautilusWindowOpenFlags flags)
+			      NautilusWindowOpenFlags flags,
+			      gboolean user_confirmation)
 {
 	ActivateParameters *parameters;
 	char *file_name;
@@ -1558,8 +1598,8 @@ nautilus_mime_activate_files (GtkWindow *parent_window,
 					   parent_window);
 
 	parameters = g_new0 (ActivateParameters, 1);
-	parameters->window_info = window_info;
-	g_object_add_weak_pointer (G_OBJECT (parameters->window_info), (gpointer *)&parameters->window_info);
+	parameters->slot_info = slot_info;
+	g_object_add_weak_pointer (G_OBJECT (parameters->slot_info), (gpointer *)&parameters->slot_info);
 	if (parent_window) {
 		parameters->parent_window = parent_window;
 		g_object_add_weak_pointer (G_OBJECT (parameters->parent_window), (gpointer *)&parameters->parent_window);
@@ -1569,6 +1609,7 @@ nautilus_mime_activate_files (GtkWindow *parent_window,
 	parameters->files = nautilus_file_list_copy (files);
 	parameters->mode = mode;
 	parameters->flags = flags;
+	parameters->user_confirmation = user_confirmation;
 
 	file_count = g_list_length (files);
 	if (file_count == 1) {
@@ -1610,7 +1651,7 @@ nautilus_mime_activate_files (GtkWindow *parent_window,
 
 void
 nautilus_mime_activate_file (GtkWindow *parent_window,
-			     NautilusWindowInfo *window_info,
+			     NautilusWindowSlotInfo *slot_info,
 			     NautilusFile *file,
 			     const char *launch_directory,
 			     NautilusWindowOpenMode mode,
@@ -1621,6 +1662,6 @@ nautilus_mime_activate_file (GtkWindow *parent_window,
 	g_return_if_fail (NAUTILUS_IS_FILE (file));
 
 	files = g_list_prepend (NULL, file);
-	nautilus_mime_activate_files (parent_window, window_info, files, launch_directory, mode, flags);
+	nautilus_mime_activate_files (parent_window, slot_info, files, launch_directory, mode, flags, FALSE);
 	g_list_free (files);
 }
