@@ -39,14 +39,7 @@
 #include <eel/eel-vfs-extensions.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkmenu.h>
-#include <gtk/gtkmenuitem.h>
-#include <gtk/gtkradiomenuitem.h>
-#include <gtk/gtkradioaction.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtkstock.h>
+#include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <libnautilus-private/nautilus-directory-background.h>
@@ -73,6 +66,12 @@
 
 #define POPUP_PATH_ICON_APPEARANCE		"/selection/Icon Appearance Items"
 
+enum 
+{
+	PROP_0,
+	PROP_COMPACT
+};
+
 typedef struct {
 	const NautilusFileSortType sort_type;
 	const char *metadata_text;
@@ -107,6 +106,8 @@ struct FMIconViewDetails
 
 	gboolean filter_by_screen;
 	int num_screens;
+
+	gboolean compact;
 };
 
 
@@ -165,6 +166,7 @@ static void                 fm_icon_view_update_click_mode            (FMIconVie
 static void                 fm_icon_view_set_directory_tighter_layout (FMIconView           *icon_view,
 								       NautilusFile         *file,
 								       gboolean              tighter_layout);
+static gboolean              fm_icon_view_supports_manual_layout      (FMIconView           *icon_view);
 static const SortCriterion *get_sort_criterion_by_sort_type           (NautilusFileSortType  sort_type);
 static void                 set_sort_criterion_by_sort_type           (FMIconView           *icon_view,
 								       NautilusFileSortType  sort_type);
@@ -176,6 +178,7 @@ static void                 preview_audio                             (FMIconVie
 								       gboolean              start_flag);
 static void                 update_layout_menus                       (FMIconView           *view);
 
+
 static void fm_icon_view_iface_init (NautilusViewIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (FMIconView, fm_icon_view, FM_TYPE_DIRECTORY_VIEW, 
@@ -186,7 +189,6 @@ static void
 fm_icon_view_destroy (GtkObject *object)
 {
 	FMIconView *icon_view;
-	GtkUIManager *ui_manager;
 
 	icon_view = FM_ICON_VIEW (object);
 
@@ -203,13 +205,6 @@ fm_icon_view_destroy (GtkObject *object)
 		icon_view->details->icons_not_positioned = NULL;
 	}
 
-	ui_manager = fm_directory_view_get_ui_manager (FM_DIRECTORY_VIEW (icon_view));
-	if (ui_manager != NULL) {
-		nautilus_ui_unmerge_ui (ui_manager,
-					&icon_view->details->icon_merge_id,
-					&icon_view->details->icon_action_group);
-	}
-
 	GTK_OBJECT_CLASS (fm_icon_view_parent_class)->destroy (object);
 }
 
@@ -248,6 +243,10 @@ get_stored_icon_position_callback (NautilusIconContainer *container,
 	g_assert (position != NULL);
 	g_assert (FM_IS_ICON_VIEW (icon_view));
 
+	if (!fm_icon_view_supports_manual_layout (icon_view)) {
+		return FALSE;
+	}
+
 	/* Doing parsing in the "C" locale instead of the one set
 	 * by the user ensures that data in the metafile is not in
 	 * a locale-specific format. It's only necessary for floating
@@ -604,6 +603,16 @@ fm_icon_view_supports_auto_layout (FMIconView *view)
 		 supports_auto_layout, (view));
 }
 
+static gboolean
+fm_icon_view_supports_manual_layout (FMIconView *view)
+{
+	g_return_val_if_fail (FM_IS_ICON_VIEW (view), FALSE);
+
+	return EEL_CALL_METHOD_WITH_RETURN_VALUE
+		(FM_ICON_VIEW_CLASS, view,
+		 supports_manual_layout, (view));
+}
+
 static gboolean
 fm_icon_view_supports_keep_aligned (FMIconView *view)
 {
@@ -624,6 +633,12 @@ fm_icon_view_supports_labels_beside_icons (FMIconView *view)
 		 supports_labels_beside_icons, (view));
 }
 
+static gboolean
+fm_icon_view_supports_tighter_layout (FMIconView *view)
+{
+	return !fm_icon_view_is_compact (view);
+}
+
 static void
 update_layout_menus (FMIconView *view)
 {
@@ -648,6 +663,9 @@ update_layout_menus (FMIconView *view)
 						      FM_ACTION_TIGHTER_LAYOUT);
 		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
 					      fm_icon_view_using_tighter_layout (view));
+		gtk_action_set_sensitive (action, fm_icon_view_supports_tighter_layout (view));
+		gtk_action_set_visible (action, fm_icon_view_supports_tighter_layout (view));
+
 		action = gtk_action_group_get_action (view->details->icon_action_group,
 						      FM_ACTION_REVERSED_ORDER);
 		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
@@ -655,6 +673,11 @@ update_layout_menus (FMIconView *view)
 		gtk_action_set_sensitive (action, is_auto_layout);
 	}
 
+	action = gtk_action_group_get_action (view->details->icon_action_group,
+					      FM_ACTION_MANUAL_LAYOUT);
+	gtk_action_set_visible (action,
+				fm_icon_view_supports_manual_layout (view));
+
 	/* Clean Up is only relevant for manual layout */
 	action = gtk_action_group_get_action (view->details->icon_action_group,
 					      FM_ACTION_CLEAN_UP);
@@ -771,7 +794,7 @@ static gboolean
 fm_icon_view_real_get_directory_sort_reversed (FMIconView *icon_view,
 					       NautilusFile *file)
 {
-	return  nautilus_file_get_boolean_metadata
+	return nautilus_file_get_boolean_metadata
 		(file,
 		 NAUTILUS_METADATA_KEY_ICON_VIEW_SORT_REVERSED,
 		 get_default_sort_in_reverse_order ());
@@ -797,7 +820,8 @@ fm_icon_view_real_set_directory_sort_reversed (FMIconView *icon_view,
 					       gboolean sort_reversed)
 {
 	nautilus_file_set_boolean_metadata
-		(file, NAUTILUS_METADATA_KEY_ICON_VIEW_SORT_REVERSED,
+		(file,
+		 NAUTILUS_METADATA_KEY_ICON_VIEW_SORT_REVERSED,
 		 get_default_sort_in_reverse_order (),
 		 sort_reversed);
 }
@@ -862,6 +886,10 @@ fm_icon_view_get_directory_auto_layout (FMIconView *icon_view,
 		return FALSE;
 	}
 
+	if (!fm_icon_view_supports_manual_layout (icon_view)) {
+		return TRUE;
+	}
+
 	return EEL_CALL_METHOD_WITH_RETURN_VALUE
 		(FM_ICON_VIEW_CLASS, icon_view,
 		 get_directory_auto_layout, (icon_view, file));
@@ -871,6 +899,8 @@ static gboolean
 fm_icon_view_real_get_directory_auto_layout (FMIconView *icon_view,
 					     NautilusFile *file)
 {
+
+
 	return nautilus_file_get_boolean_metadata
 		(file, NAUTILUS_METADATA_KEY_ICON_VIEW_AUTO_LAYOUT, !get_default_directory_manual_layout ());
 }
@@ -880,7 +910,8 @@ fm_icon_view_set_directory_auto_layout (FMIconView *icon_view,
 					NautilusFile *file,
 					gboolean auto_layout)
 {
-	if (!fm_icon_view_supports_auto_layout (icon_view)) {
+	if (!fm_icon_view_supports_auto_layout (icon_view) ||
+	    !fm_icon_view_supports_manual_layout (icon_view)) {
 		return;
 	}
 
@@ -893,6 +924,10 @@ fm_icon_view_real_set_directory_auto_layout (FMIconView *icon_view,
 					     NautilusFile *file,
 					     gboolean auto_layout)
 {
+	if (!fm_icon_view_supports_manual_layout (icon_view)) {
+		return;
+	}
+
 	nautilus_file_set_boolean_metadata
 		(file, NAUTILUS_METADATA_KEY_ICON_VIEW_AUTO_LAYOUT,
 		 !get_default_directory_manual_layout (),
@@ -929,6 +964,10 @@ static gboolean
 fm_icon_view_real_get_directory_tighter_layout (FMIconView *icon_view,
 						NautilusFile *file)
 {
+	if (!fm_icon_view_supports_tighter_layout (icon_view)) {
+		return FALSE;
+	}
+
 	return nautilus_file_get_boolean_metadata
 		(file,
 		 NAUTILUS_METADATA_KEY_ICON_VIEW_TIGHTER_LAYOUT,
@@ -949,6 +988,10 @@ fm_icon_view_real_set_directory_tighter_layout (FMIconView *icon_view,
 						NautilusFile *file,
 						gboolean tighter_layout)
 {
+	if (!fm_icon_view_supports_tighter_layout (icon_view)) {
+		return;
+	}
+
 	nautilus_file_set_boolean_metadata
 		(file, NAUTILUS_METADATA_KEY_ICON_VIEW_TIGHTER_LAYOUT,
 		 get_default_directory_tighter_layout (),
@@ -963,6 +1006,14 @@ real_supports_auto_layout (FMIconView *view)
 	return TRUE;
 }
 
+static gboolean
+real_supports_manual_layout (FMIconView *view)
+{
+	g_return_val_if_fail (FM_IS_ICON_VIEW (view), FALSE);
+
+	return !fm_icon_view_is_compact (view);
+}
+
 static gboolean
 real_supports_keep_aligned (FMIconView *view)
 {
@@ -1026,9 +1077,11 @@ get_sort_criterion_by_sort_type (NautilusFileSortType sort_type)
 }
 
 static NautilusZoomLevel default_zoom_level = NAUTILUS_ZOOM_LEVEL_STANDARD;
+static NautilusZoomLevel default_compact_zoom_level = NAUTILUS_ZOOM_LEVEL_STANDARD;
+#define DEFAULT_ZOOM_LEVEL(icon_view) icon_view->details->compact ? default_compact_zoom_level : default_zoom_level
 
 static NautilusZoomLevel
-get_default_zoom_level (void)
+get_default_zoom_level (FMIconView *icon_view)
 {
 	static gboolean auto_storage_added = FALSE;
 
@@ -1036,9 +1089,11 @@ get_default_zoom_level (void)
 		auto_storage_added = TRUE;
 		eel_preferences_add_auto_enum (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
 					       (int *) &default_zoom_level);
+		eel_preferences_add_auto_enum (NAUTILUS_PREFERENCES_COMPACT_VIEW_DEFAULT_ZOOM_LEVEL,
+					       (int *) &default_compact_zoom_level);
 	}
 
-	return CLAMP (default_zoom_level, NAUTILUS_ZOOM_LEVEL_SMALLEST, NAUTILUS_ZOOM_LEVEL_LARGEST);
+	return CLAMP (DEFAULT_ZOOM_LEVEL(icon_view), NAUTILUS_ZOOM_LEVEL_SMALLEST, NAUTILUS_ZOOM_LEVEL_LARGEST);
 }
 
 static void
@@ -1047,7 +1102,8 @@ set_labels_beside_icons (FMIconView *icon_view)
 	gboolean labels_beside;
 
 	if (fm_icon_view_supports_labels_beside_icons (icon_view)) {
-		labels_beside = eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ICON_VIEW_LABELS_BESIDE_ICONS);
+		labels_beside = fm_icon_view_is_compact (icon_view) ||
+				eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ICON_VIEW_LABELS_BESIDE_ICONS);
 		
 		if (labels_beside) {
 			nautilus_icon_container_set_label_position
@@ -1061,6 +1117,17 @@ set_labels_beside_icons (FMIconView *icon_view)
 	}
 }
 
+static void
+set_columns_same_width (FMIconView *icon_view)
+{
+	gboolean all_columns_same_width;
+
+	if (fm_icon_view_is_compact (icon_view)) {
+		all_columns_same_width = eel_preferences_get_boolean (NAUTILUS_PREFERENCES_COMPACT_VIEW_ALL_COLUMNS_SAME_WIDTH);
+		nautilus_icon_container_set_all_columns_same_width (get_icon_container (icon_view), all_columns_same_width);
+	}
+}
+
 static void
 fm_icon_view_begin_loading (FMDirectoryView *view)
 {
@@ -1105,10 +1172,18 @@ fm_icon_view_begin_loading (FMDirectoryView *view)
 	
 	/* Set up the zoom level from the metadata. */
 	if (fm_directory_view_supports_zooming (FM_DIRECTORY_VIEW (icon_view))) {
-		level = nautilus_file_get_integer_metadata
-			(file, 
-			 NAUTILUS_METADATA_KEY_ICON_VIEW_ZOOM_LEVEL, 
-			 get_default_zoom_level ());
+		if (icon_view->details->compact) {
+			level = nautilus_file_get_integer_metadata
+				(file, 
+				 NAUTILUS_METADATA_KEY_COMPACT_VIEW_ZOOM_LEVEL, 
+				 get_default_zoom_level (icon_view));
+		} else {
+			level = nautilus_file_get_integer_metadata
+				(file, 
+				 NAUTILUS_METADATA_KEY_ICON_VIEW_ZOOM_LEVEL, 
+				 get_default_zoom_level (icon_view));
+		}
+
 		fm_icon_view_set_zoom_level (icon_view, level, TRUE);
 	}
 
@@ -1131,6 +1206,7 @@ fm_icon_view_begin_loading (FMDirectoryView *view)
 		 fm_icon_view_get_directory_tighter_layout (icon_view, file));
 
 	set_labels_beside_icons (icon_view);
+	set_columns_same_width (icon_view);
 
 	/* We must set auto-layout last, because it invokes the layout_changed 
 	 * callback, which works incorrectly if the other layout criteria are
@@ -1179,17 +1255,27 @@ fm_icon_view_set_zoom_level (FMIconView *view,
 		return;
 	}
 
-	nautilus_file_set_integer_metadata
-		(fm_directory_view_get_directory_as_file (FM_DIRECTORY_VIEW (view)), 
-		 NAUTILUS_METADATA_KEY_ICON_VIEW_ZOOM_LEVEL, 
-		 get_default_zoom_level (),
-		 new_level);
+	if (view->details->compact) {
+		nautilus_file_set_integer_metadata
+			(fm_directory_view_get_directory_as_file (FM_DIRECTORY_VIEW (view)), 
+			 NAUTILUS_METADATA_KEY_COMPACT_VIEW_ZOOM_LEVEL, 
+			 get_default_zoom_level (view),
+			 new_level);
+	} else {
+		nautilus_file_set_integer_metadata
+			(fm_directory_view_get_directory_as_file (FM_DIRECTORY_VIEW (view)), 
+			 NAUTILUS_METADATA_KEY_ICON_VIEW_ZOOM_LEVEL, 
+			 get_default_zoom_level (view),
+			 new_level);
+	}
 
 	nautilus_icon_container_set_zoom_level (icon_container, new_level);
 
 	g_signal_emit_by_name (view, "zoom_level_changed");
 	
-	fm_directory_view_update_menus (FM_DIRECTORY_VIEW (view));
+	if (fm_directory_view_get_active (FM_DIRECTORY_VIEW (view))) {
+		fm_directory_view_update_menus (FM_DIRECTORY_VIEW (view));
+	}
 }
 
 static void
@@ -1215,7 +1301,7 @@ fm_icon_view_zoom_to_level (FMDirectoryView *view,
 {
 	FMIconView *icon_view;
 
-	g_return_if_fail (FM_IS_ICON_VIEW (view));
+	g_assert (FM_IS_ICON_VIEW (view));
 
 	icon_view = FM_ICON_VIEW (view);
 	fm_icon_view_set_zoom_level (icon_view, zoom_level, FALSE);
@@ -1224,8 +1310,13 @@ fm_icon_view_zoom_to_level (FMDirectoryView *view,
 static void
 fm_icon_view_restore_default_zoom_level (FMDirectoryView *view)
 {
+	FMIconView *icon_view;
+
+	g_return_if_fail (FM_IS_ICON_VIEW (view));
+
+	icon_view = FM_ICON_VIEW (view);
 	fm_directory_view_zoom_to_level
-		(view, get_default_zoom_level ());
+		(view, get_default_zoom_level (icon_view));
 }
 
 static gboolean 
@@ -1422,7 +1513,7 @@ fm_icon_view_start_renaming_file (FMDirectoryView *view,
 static const GtkActionEntry icon_view_entries[] = {
   /* name, stock id, label */  { "Arrange Items", NULL, N_("Arran_ge Items") }, 
   /* name, stock id */         { "Stretch", NULL,
-  /* label, accelerator */       N_("Str_etch Icon"), NULL,
+  /* label, accelerator */       N_("Stretc_h Icon..."), NULL,
   /* tooltip */                  N_("Make the selected icon stretchable"),
                                  G_CALLBACK (action_stretch_callback) },
   /* name, stock id */         { "Unstretch", NULL,
@@ -1549,6 +1640,24 @@ fm_icon_view_merge_menus (FMDirectoryView *view)
 	update_layout_menus (icon_view);
 }
 
+static void
+fm_icon_view_unmerge_menus (FMDirectoryView *view)
+{
+	FMIconView *icon_view;
+	GtkUIManager *ui_manager;
+
+	icon_view = FM_ICON_VIEW (view);
+
+	FM_DIRECTORY_VIEW_CLASS (fm_icon_view_parent_class)->unmerge_menus (view);
+
+	ui_manager = fm_directory_view_get_ui_manager (view);
+	if (ui_manager != NULL) {
+		nautilus_ui_unmerge_ui (ui_manager,
+					&icon_view->details->icon_merge_id,
+					&icon_view->details->icon_action_group);
+	}
+}
+
 static void
 fm_icon_view_update_menus (FMDirectoryView *view)
 {
@@ -1673,6 +1782,15 @@ fm_icon_view_set_selection (FMDirectoryView *view, GList *selection)
 		(get_icon_container (FM_ICON_VIEW (view)), selection);
 }
 
+static void
+fm_icon_view_invert_selection (FMDirectoryView *view)
+{
+	g_return_if_fail (FM_IS_ICON_VIEW (view));
+
+	nautilus_icon_container_invert_selection
+		(get_icon_container (FM_ICON_VIEW (view)));
+}
+
 static gboolean
 fm_icon_view_using_manual_layout (FMDirectoryView *view)
 {
@@ -1701,7 +1819,8 @@ icon_container_activate_callback (NautilusIconContainer *container,
 
 	fm_directory_view_activate_files (FM_DIRECTORY_VIEW (icon_view),
 					  file_list, 
-					  NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE, 0);
+					  NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE, 0,
+					  TRUE);
 }
 
 static void
@@ -1709,14 +1828,50 @@ icon_container_activate_alternate_callback (NautilusIconContainer *container,
 					    GList *file_list,
 					    FMIconView *icon_view)
 {
+	GdkEvent *event;
+	GdkEventButton *button_event;
+	GdkEventKey *key_event;
+	gboolean open_in_tab;
+	NautilusWindowInfo *window_info;
+	NautilusWindowOpenFlags flags;
+
 	g_assert (FM_IS_ICON_VIEW (icon_view));
 	g_assert (container == get_icon_container (icon_view));
 
+	open_in_tab = FALSE;
+
+	window_info = fm_directory_view_get_nautilus_window (FM_DIRECTORY_VIEW (icon_view));
+
+	if (nautilus_window_info_get_window_type (window_info) == NAUTILUS_WINDOW_NAVIGATION) {
+		event = gtk_get_current_event ();
+		if (event->type == GDK_BUTTON_PRESS ||
+		    event->type == GDK_BUTTON_RELEASE ||
+		    event->type == GDK_2BUTTON_PRESS ||
+		    event->type == GDK_3BUTTON_PRESS) {
+			button_event = (GdkEventButton *) event;
+			open_in_tab = (button_event->state & GDK_SHIFT_MASK) == 0;
+		} else if (event->type == GDK_KEY_PRESS ||
+			   event->type == GDK_KEY_RELEASE) {
+			key_event = (GdkEventKey *) event;
+			open_in_tab = !((key_event->state & GDK_SHIFT_MASK) != 0 &&
+				       (key_event->state & GDK_CONTROL_MASK) != 0);
+		} else {
+			open_in_tab = TRUE;
+		}
+	}
+
+	flags = NAUTILUS_WINDOW_OPEN_FLAG_CLOSE_BEHIND;
+	if (open_in_tab && eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ENABLE_TABS)) {
+		flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB;
+	} else {
+		flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW;
+	}
+
 	fm_directory_view_activate_files (FM_DIRECTORY_VIEW (icon_view), 
 					  file_list, 
 					  NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
-					  NAUTILUS_WINDOW_OPEN_FLAG_CLOSE_BEHIND |
-					  NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW);
+					  flags,
+					  TRUE);
 }
 
 static void
@@ -1957,8 +2112,8 @@ icon_container_preview_callback (NautilusIconContainer *container,
 			file_name = nautilus_file_get_display_name (file);
 			message = g_strdup_printf (_("pointing at \"%s\""), file_name);
 			g_free (file_name);
-			nautilus_window_info_set_status
-				(fm_directory_view_get_nautilus_window (FM_DIRECTORY_VIEW (icon_view)),
+			nautilus_window_slot_info_set_status
+				(fm_directory_view_get_nautilus_window_slot (FM_DIRECTORY_VIEW (icon_view)),
 				 message);
 			g_free (message);
 		} else {
@@ -2135,7 +2290,8 @@ icon_position_changed_callback (NautilusIconContainer *container,
 	 * idle call, because we'd have to keep track of potentially multiple
 	 * sets of file/geometry info.
 	 */
-	if (icon_view->details->react_to_icon_change_idle_id == 0) {
+	if (fm_directory_view_get_active (FM_DIRECTORY_VIEW (icon_view)) &&
+	    icon_view->details->react_to_icon_change_idle_id == 0) {
                 icon_view->details->react_to_icon_change_idle_id
                         = g_idle_add (fm_icon_view_react_to_icon_change_idle_callback,
 				      icon_view);
@@ -2348,10 +2504,16 @@ default_zoom_level_changed_callback (gpointer callback_data)
 
 	if (fm_directory_view_supports_zooming (FM_DIRECTORY_VIEW (icon_view))) {
 		file = fm_directory_view_get_directory_as_file (FM_DIRECTORY_VIEW (icon_view));
-		
-		level = nautilus_file_get_integer_metadata (file, 
-							    NAUTILUS_METADATA_KEY_ICON_VIEW_ZOOM_LEVEL, 
-							    get_default_zoom_level ());
+
+		if (fm_icon_view_is_compact (icon_view)) {
+			level = nautilus_file_get_integer_metadata (file, 
+								    NAUTILUS_METADATA_KEY_COMPACT_VIEW_ZOOM_LEVEL, 
+								    get_default_zoom_level (icon_view));
+		} else {
+			level = nautilus_file_get_integer_metadata (file, 
+								    NAUTILUS_METADATA_KEY_ICON_VIEW_ZOOM_LEVEL, 
+								    get_default_zoom_level (icon_view));
+		}
 		fm_directory_view_zoom_to_level (FM_DIRECTORY_VIEW (icon_view), level);
 	}
 }
@@ -2368,6 +2530,19 @@ labels_beside_icons_changed_callback (gpointer callback_data)
 	set_labels_beside_icons (icon_view);
 }
 
+static void
+all_columns_same_width_changed_callback (gpointer callback_data)
+{
+	FMIconView *icon_view;
+
+	g_assert (FM_IS_ICON_VIEW (callback_data));
+
+	icon_view = FM_ICON_VIEW (callback_data);
+
+	set_columns_same_width (icon_view);
+}
+
+
 static void
 fm_icon_view_sort_directories_first_changed (FMDirectoryView *directory_view)
 {
@@ -2408,6 +2583,9 @@ icon_view_move_copy_items (NautilusIconContainer *container,
 			   int x, int y,
 			   FMDirectoryView *view)
 {
+	nautilus_clipboard_clear_if_colliding_uris (GTK_WIDGET (view),
+						    item_uris,
+						    fm_directory_view_get_copied_files_atom (view));
 	fm_directory_view_move_copy_items (item_uris, relative_item_points, target_dir,
 		copy_action, x, y, view);
 }
@@ -2556,6 +2734,34 @@ icon_view_scroll_to_file (NautilusView *view,
 	}
 }
 
+static void
+fm_icon_view_set_property (GObject         *object,
+			   guint            prop_id,
+			   const GValue    *value,
+			   GParamSpec      *pspec)
+{
+	FMIconView *icon_view;
+  
+	icon_view = FM_ICON_VIEW (object);
+
+	switch (prop_id)  {
+		case PROP_COMPACT:
+			icon_view->details->compact = g_value_get_boolean (value);
+			if (icon_view->details->compact) {
+				nautilus_icon_container_set_layout_mode (get_icon_container (icon_view),
+									 NAUTILUS_ICON_LAYOUT_T_B_L_R);
+				nautilus_icon_container_set_forced_icon_size (get_icon_container (icon_view),
+									      NAUTILUS_ICON_SIZE_SMALLEST);
+			}
+		break;
+
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+
 
 static void
 fm_icon_view_class_init (FMIconViewClass *klass)
@@ -2564,6 +2770,7 @@ fm_icon_view_class_init (FMIconViewClass *klass)
 
 	fm_directory_view_class = FM_DIRECTORY_VIEW_CLASS (klass);
 
+	G_OBJECT_CLASS (klass)->set_property = fm_icon_view_set_property;
 	G_OBJECT_CLASS (klass)->finalize = fm_icon_view_finalize;
 
 	GTK_OBJECT_CLASS (klass)->destroy = fm_icon_view_destroy;
@@ -2592,6 +2799,7 @@ fm_icon_view_class_init (FMIconViewClass *klass)
 	fm_directory_view_class->reveal_selection = fm_icon_view_reveal_selection;
 	fm_directory_view_class->select_all = fm_icon_view_select_all;
 	fm_directory_view_class->set_selection = fm_icon_view_set_selection;
+	fm_directory_view_class->invert_selection = fm_icon_view_invert_selection;
 	fm_directory_view_class->compare_files = compare_files;
 	fm_directory_view_class->zoom_to_level = fm_icon_view_zoom_to_level;
 	fm_directory_view_class->get_zoom_level = fm_icon_view_get_zoom_level;
@@ -2600,6 +2808,7 @@ fm_icon_view_class_init (FMIconViewClass *klass)
         fm_directory_view_class->emblems_changed = fm_icon_view_emblems_changed;
         fm_directory_view_class->image_display_policy_changed = fm_icon_view_image_display_policy_changed;
         fm_directory_view_class->merge_menus = fm_icon_view_merge_menus;
+        fm_directory_view_class->unmerge_menus = fm_icon_view_unmerge_menus;
         fm_directory_view_class->sort_directories_first_changed = fm_icon_view_sort_directories_first_changed;
         fm_directory_view_class->start_renaming_file = fm_icon_view_start_renaming_file;
         fm_directory_view_class->text_attribute_names_changed = fm_icon_view_text_attribute_names_changed;
@@ -2609,6 +2818,7 @@ fm_icon_view_class_init (FMIconViewClass *klass)
 
 	klass->clean_up = fm_icon_view_real_clean_up;
 	klass->supports_auto_layout = real_supports_auto_layout;
+	klass->supports_manual_layout = real_supports_manual_layout;
 	klass->supports_keep_aligned = real_supports_keep_aligned;
 	klass->supports_labels_beside_icons = real_supports_labels_beside_icons;
         klass->get_directory_auto_layout = fm_icon_view_real_get_directory_auto_layout;
@@ -2619,6 +2829,16 @@ fm_icon_view_class_init (FMIconViewClass *klass)
         klass->set_directory_sort_by = fm_icon_view_real_set_directory_sort_by;
         klass->set_directory_sort_reversed = fm_icon_view_real_set_directory_sort_reversed;
         klass->set_directory_tighter_layout = fm_icon_view_real_set_directory_tighter_layout;
+
+	g_object_class_install_property (G_OBJECT_CLASS (klass),
+					 PROP_COMPACT,
+					 g_param_spec_boolean ("compact",
+							       "Compact",
+							       "Whether this view provides a compact listing",
+							       FALSE,
+							       G_PARAM_WRITABLE |
+							       G_PARAM_CONSTRUCT_ONLY));
+
 }
 
 static const char *
@@ -2627,6 +2847,11 @@ fm_icon_view_get_id (NautilusView *view)
 	if (FM_IS_DESKTOP_ICON_VIEW (view)) {
 		return FM_DESKTOP_ICON_VIEW_ID;
 	}
+
+	if (fm_icon_view_is_compact (FM_ICON_VIEW (view))) {
+		return FM_COMPACT_VIEW_ID;
+	}
+
 	return FM_ICON_VIEW_ID;
 }
 
@@ -2687,6 +2912,13 @@ fm_icon_view_init (FMIconView *icon_view)
 						  labels_beside_icons_changed_callback,
 						  icon_view, G_OBJECT (icon_view));
 
+	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_COMPACT_VIEW_DEFAULT_ZOOM_LEVEL,
+						  default_zoom_level_changed_callback,
+						  icon_view, G_OBJECT (icon_view));
+	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_COMPACT_VIEW_ALL_COLUMNS_SAME_WIDTH,
+						  all_columns_same_width_changed_callback,
+						  icon_view, G_OBJECT (icon_view));
+
 	g_signal_connect_object (get_icon_container (icon_view), "handle_netscape_url",
 				 G_CALLBACK (icon_view_handle_netscape_url), icon_view, 0);
 	g_signal_connect_object (get_icon_container (icon_view), "handle_uri_list",
@@ -2696,11 +2928,28 @@ fm_icon_view_init (FMIconView *icon_view)
 }
 
 static NautilusView *
-fm_icon_view_create (NautilusWindowInfo *window)
+fm_icon_view_create (NautilusWindowSlotInfo *slot)
 {
 	FMIconView *view;
 
-	view = g_object_new (FM_TYPE_ICON_VIEW, "window", window, NULL);
+	view = g_object_new (FM_TYPE_ICON_VIEW,
+			     "window-slot", slot,
+			     "compact", FALSE,
+			     NULL);
+	g_object_ref (view);
+	gtk_object_sink (GTK_OBJECT (view));
+	return NAUTILUS_VIEW (view);
+}
+
+static NautilusView *
+fm_compact_view_create (NautilusWindowSlotInfo *slot)
+{
+	FMIconView *view;
+
+	view = g_object_new (FM_TYPE_ICON_VIEW,
+			     "window-slot", slot,
+			     "compact", TRUE,
+			     NULL);
 	g_object_ref (view);
 	gtk_object_sink (GTK_OBJECT (view));
 	return NAUTILUS_VIEW (view);
@@ -2727,11 +2976,21 @@ fm_icon_view_supports_uri (const char *uri,
 	return FALSE;
 }
 
+#define TRANSLATE_VIEW_INFO(view_info) \
+	view_info.view_combo_label = _(view_info.view_combo_label); \
+	view_info.view_menu_label_with_mnemonic = _(view_info.view_menu_label_with_mnemonic); \
+	view_info.error_label = _(view_info.error_label); \
+	view_info.startup_error_label = _(view_info.startup_error_label); \
+	view_info.display_location_label = _(view_info.display_location_label); \
+	
+
 static NautilusViewInfo fm_icon_view = {
 	FM_ICON_VIEW_ID,
-	N_("Icons"),
-	N_("View as Icons"),
-	N_("View as _Icons"),
+	/* translators: this is used in the view selection dropdown
+	 * of navigation windows and in the preferences dialog */
+	N_("Icon View"),
+	/* translators: this is used in the view menu */
+	N_("_Icons"),
 	N_("The icon view encountered an error."),
 	N_("The icon view encountered an error while starting up."),
 	N_("Display this location with the icon view."),
@@ -2739,15 +2998,37 @@ static NautilusViewInfo fm_icon_view = {
 	fm_icon_view_supports_uri
 };
 
+static NautilusViewInfo fm_compact_view = {
+	FM_COMPACT_VIEW_ID,
+	/* translators: this is used in the view selection dropdown
+	 * of navigation windows and in the preferences dialog */
+	N_("Compact View"),
+	/* translators: this is used in the view menu */
+	N_("_Compact"),
+	N_("The compact view encountered an error."),
+	N_("The compact view encountered an error while starting up."),
+	N_("Display this location with the compact view."),
+	fm_compact_view_create,
+	fm_icon_view_supports_uri
+};
+
+gboolean
+fm_icon_view_is_compact (FMIconView *view)
+{
+	return view->details->compact;
+}
+
 void
 fm_icon_view_register (void)
 {
-	fm_icon_view.label = _(fm_icon_view.label);
-	fm_icon_view.view_as_label = _(fm_icon_view.view_as_label);
-	fm_icon_view.view_as_label_with_mnemonic = _(fm_icon_view.view_as_label_with_mnemonic);
-	fm_icon_view.error_label = _(fm_icon_view.error_label);
-	fm_icon_view.startup_error_label = _(fm_icon_view.startup_error_label);
-	fm_icon_view.display_location_label = _(fm_icon_view.display_location_label);
-	
+	TRANSLATE_VIEW_INFO (fm_icon_view)
 	nautilus_view_factory_register (&fm_icon_view);
 }
+
+void
+fm_compact_view_register (void)
+{
+	TRANSLATE_VIEW_INFO (fm_compact_view)
+	nautilus_view_factory_register (&fm_compact_view);
+}
+
