@@ -72,7 +72,6 @@ void                 nautilus_application_startup           (NautilusApplication
 							     gboolean             restart_shell,
 							     gboolean             no_default_window,
 							     gboolean             no_desktop,
-							     gboolean             do_first_time_druid_check,
 							     gboolean             browser_window,
 							     const char          *startup_id,
 							     const char          *default_geometry,
@@ -105,4 +104,5 @@ void                 nautilus_application_open_desktop      (NautilusApplication
 void                 nautilus_application_close_desktop     (void);
 void                 nautilus_application_load_session      (NautilusApplication *application,
 							     const char *filename); 
+char *               nautilus_application_save_session_to_file (void);
 #endif /* NAUTILUS_APPLICATION_H */
